'use strict';
/*
 * Transforms pages[0..length].content from markdown to HTML
 */

var marked = require('marked');
var renderer = new marked.Renderer();
var highlightjs = require('highlight.js');
var path = require('path');
var jobs = require('./jobs');
var youtube = require('./utils/youtube.js');

var toURL = function(url) {
  return jobs.run('toURL', url);
}

renderer.heading = function(text, level, raw) {
  return '<h'  +
    (level+2)  +
    ' id="'    +
    toURL(raw) +
    '">'       +
    text       +
    '</h'      +
    (level+2)  +
    '>\n';
}

renderer.paragraph = function(text) {
  if(text.match('\\<img')) return '<div class="has-image">'+text+'</div>\n';
  else return '<p>'+text+'</p>\n';
}

var paramInHrefParser = function(href) {
  var sepIndex = href.indexOf(' =');
  var ret = {
    href: href,
    options: {}
  }
  if(sepIndex < 0) {
    return ret;
  } else {
    ret.href = href.substr(0, sepIndex);
    var options = href.substr(sepIndex+2);
    options = options // split by ',' but not '\,'
      .replace('a,','\u0007')
      .split(',')
      .map(function(e) {return e.replace('\u0007','a,')});
    options.forEach(function(option) {
      var index = option.indexOf('=');
      if(index < 0) {
        ret.options[option] = true;
      } else {
        var key = option.substr(0,index);
        var value = option.substr(index+1);
        ret.options[key] = value;
      }
    });
    return ret;
  }
}

var renderYoutube = function(href, options) {
  var info = youtube.info(href);
  let out = '<div class="video-container"><div class="videodiv">';
  out += '<iframe src="https://www.youtube.com/embed/'+info.videoid+
  '?modestbranding&start='+info.time+'" frameborder="0" allowfullscreen></iframe>'
  out += '</div></div>'

  return out;
}

var renderImage = function(href, title, text, curFilename) {
  var parsed = paramInHrefParser(href);
  href = parsed.href;
  var options = parsed.options;

  for(let option in options) {
    if( !options.hasOwnProperty(option) ) continue;

    if(option.match(/x[0-9]+/) || option.match(/[0-9]+x[0-9]*/)) { // 123x or x123 or 123x123
      options.size = option.split('x');
      if(options.size !== undefined) {
        if(options.size[0].length > 0) {
          options.width = options.size[0];
        }
        if(options.size.length > 1 && options.size[1].length > 0) {
          options.height = options.size[1];
        }
      }
    }
  }

  if(youtube.isVideo(href)) {
    return renderYoutube(href, options);
  } else {
    var rel = path.relative(process.cwd(),path.resolve(curFilename,href));

    var out = '<img src="' + href + '" alt="' + text + '"';
    if(options.width) {
      out += ' width="' + options.width + '"';
    }
    if(options.height) {
      out += ' height="' + options.height + '"';
    }
    if (title) {
      out += ' title="' + title + '"';
    }

    out += '>';
    if(!options.nolightbox) { // if lightbox
      var a = '<a href="'+href+'"';
      a += ' data-lightbox="group"';
      a += ' data-title="' + text + '"';
      a += '>';
      out = a+out+'</a>';
    }
    return out;
  }
}

renderer._link = renderer.link;

marked.setOptions({
  renderer: renderer,
  gfm: true,
  tables: true,
  breaks: false,
  pedantic: false,
  sanitize: false,
  smartLists: true,
  smartypants: false,
});

module.exports = function(obj) {
  console.log('Build step: Markdown');
  var options = obj.config.options;
  renderer.link = function(href, title, text) {
    if(options.httponly) {
      if(href.match(/^https:\/\/ok1kvk.cz/))
        href = href.replace('https://', 'http://');
    }
    if(options.httpsonly) {
      if(href.match(/^http:\/\/ok1kvk.cz/))
        href = href.replace('http://', 'https://');
    }

    return renderer._link(href,title,text);
  };

  obj.pages.forEach(function(article) {
    renderer.image = function(href, title, text) {
      return renderImage(href, title, text, article.filename);
    };

    article.content = marked(article.content, {
      highlight: function (code) {
        var h;
        if(highlightjs.getLanguage(article.metadata.lang))
          h = highlightjs.highlight(article.metadata.lang, code, true);
        else
          h = highlightjs.highlightAuto(code);
        return h.value;
      },
      renderer: renderer
    });
  });
  return Promise.resolve(obj);
}

module.exports._test = {
  renderImage: renderImage,
  paramInHrefParser: paramInHrefParser,
  renderYoutube: renderYoutube
};
